--*****************************************************************************
--*  Copyright (C) 2012 by Michael Fischer
--*
--*  All rights reserved.
--*
--*  Redistribution and use in source and binary forms, with or without 
--*  modification, are permitted provided that the following conditions 
--*  are met:
--*  
--*  1. Redistributions of source code must retain the above copyright 
--*     notice, this list of conditions and the following disclaimer.
--*  2. Redistributions in binary form must reproduce the above copyright
--*     notice, this list of conditions and the following disclaimer in the 
--*     documentation and/or other materials provided with the distribution.
--*  3. Neither the name of the author nor the names of its contributors may 
--*     be used to endorse or promote products derived from this software 
--*     without specific prior written permission.
--*
--*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
--*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
--*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
--*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
--*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
--*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
--*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
--*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
--*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
--*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
--*  SUCH DAMAGE.
--*
--*****************************************************************************
--*  History:
--*
--*  22.08.2012  mifi  First Version, tested with an Altera DE1 board.
--*****************************************************************************


--*****************************************************************************
--*  DEFINE: Library                                                          *
--*****************************************************************************

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


--*****************************************************************************
--*  DEFINE: Entity                                                           *
--*****************************************************************************

entity pio_spi_if is
	port (
          -- Avalon Slave bus
		    clk        : in  std_logic := '0';
		    reset      : in  std_logic := '0';
          chipselect : in  std_logic := '0';
		    address    : in  std_logic_vector(1 downto 0) := (others => '0');
          write      : in  std_logic := '0';
          writedata  : in  std_logic_vector(31 downto 0) := (others => '0');
          read       : in  std_logic := '0';
          readdata   : out std_logic_vector(31 downto 0);
      
          -- External bus
          cs         : out std_logic;
          sclk       : out std_logic;
          mosi       : out std_logic;
          miso       : in  std_logic := '0'
	     );
end entity pio_spi_if;

--*****************************************************************************
--*  DEFINE: Architecture                                                     *
--*****************************************************************************

architecture syn of pio_spi_if is

   --
   -- Define all constants here
   --
   
   constant ADDR_DATA : integer := 0; -- Read / Write
   constant ADDR_SET  : integer := 1; -- Write
   constant ADDR_CLR  : integer := 2; -- Write


   --
   -- Define all components which are included here
   --

   
   --
   -- Define all local signals (like static data) here
   --
   
   signal data     : std_logic_vector(31 downto 0) := (others => '0');
   signal read_mux : std_logic_vector(31 downto 0) := (others => '0');  

begin

   ----------------------------------------------
   -- Register File
   ----------------------------------------------

   process(clk)
   begin
      if rising_edge(clk) then
         if (write = '1') then
         
            if (ADDR_DATA = unsigned(address)) then   -- ADDR_DATA
               data <= writedata;
            end if;             
            
            if (ADDR_SET = unsigned(address)) then    -- ADDR_SET
               data <= data OR writedata;
            end if;             
            
            if (ADDR_CLR = unsigned(address)) then    -- ADDR_CLR
               data <= data AND (NOT writedata);
            end if;             
            
         end if;
      end if; 
   end process;


   ----------------------------------------------
   -- Read Multiplexer
   ----------------------------------------------
     
   read_mux <= std_logic_vector'("0000000000000000000000000000000") & miso;

   process(clk)
   begin
      if rising_edge(clk) then
         if (read = '1') then
            readdata <= read_mux;
         end if;
      end if;
   end process;      


   ----------------------------------------------
   -- External output
   ----------------------------------------------
   
   cs   <= data(0);
   sclk <= data(1);
   mosi <= data(2);

end architecture syn;

-- *** EOF ***
